/*============================================================================
  Summary:  Demonstrates Forced Parameterization in SQL Server.
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more scripts and sample code, check out 
    http://www.SQLpassion.at

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Clears the Plan Cache
DBCC FREEPROCCACHE
GO

-- Create a copy from the Sales.SalesOrderHeader table
SELECT * INTO Sales.SalesOrderHeader2 FROM Sales.SalesOrderHeader
GO
 
-- Create a Non-Clustered Index on the CustomerID column
CREATE NONCLUSTERED INDEX idx_CustomerID ON Sales.SalesOrderHeader2(CustomerID)
GO

-- "Patch" the data in some way, so that the content of the column "CustomerID" is not evenly distributed across the whole table
UPDATE Sales.SalesOrderHeader2
SET CustomerID = 29675
WHERE SalesOrderID < 60000
GO

-- Each of the following queries gets its unique Execution Plan in the Plan Cache.
-- Each Execution Plan is optimized for the provided parameter value.
-- 780 Logical Reads
SELECT * FROM Sales.SalesOrderHeader2
WHERE CustomerID = 29675 -- Table Scan, returns 16343 records
GO

-- 3 Logical Reads
SELECT * FROM Sales.SalesOrderHeader2
WHERE CustomerID = 22943 -- Index Seek, returns 1 record
GO

-- 3 Logical Reads
SELECT * FROM Sales.SalesOrderHeader2
WHERE CustomerID = 12875 -- Index Seek, returns 1 record
GO

-- Let's now activate Forced Parameterization on the AdventureWorks2008R2 database
ALTER DATABASE AdventureWorks2012 SET PARAMETERIZATION FORCED
GO

-- Clears the Plan Cache
DBCC FREEPROCCACHE
GO

-- Execute the same queries again
-- ...

-- Table Scan, because the result is not selective enough
-- 780 Logical Reads
SELECT * FROM Sales.SalesOrderHeader2
WHERE CustomerID = 29675 -- Table Scan, returns 16343 records
GO

-- Table Scan, because the parametrized version of the Execution Plan is REUSED!
-- 780 Logical Reads!
SELECT * FROM Sales.SalesOrderHeader2
WHERE CustomerID = 22943 -- Table Scan, returns 1 record
GO

-- Table Scan, because the parametrized version of the Execution Plan is REUSED!
-- 780 Logical Reads!
SELECT * FROM Sales.SalesOrderHeader2
WHERE CustomerID = 12875 -- Table Scan, returns 1 record
GO

-- Clears the Plan Cache
DBCC FREEPROCCACHE
GO

-- Execute the same queries in a different order again
-- ...

-- Index Seek, because the result is selective enough
-- 3 Logical Reads
SELECT * FROM Sales.SalesOrderHeader2
WHERE CustomerID = 22943 -- Index Seek, returns 1 record
GO

-- Index Seek, because the parametrized version of the Execution Plan is REUSED!
-- 3 Logical Reads
SELECT * FROM Sales.SalesOrderHeader2
WHERE CustomerID = 12875 -- Index Seek, returns 1 record
GO

-- Index Seek, because the parametrized version of the Execution Plan is REUSED!
-- 16416 Logical Reads!!!
-- The "Parameter Compiled Value" and "Parameter Runtime Value" are also different!
SELECT * FROM Sales.SalesOrderHeader2
WHERE CustomerID = 29675 -- Index Seek, returns 16343 records
GO

-- Enabling Forced Parametrization is not always the best thing!!!
ALTER DATABASE AdventureWorks2012 SET PARAMETERIZATION SIMPLE
GO